!--------------------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------------------

SUBROUTINE ProjectLake(NP,PeakLon,PeakLat, IStat, &   ! IN
                       RFrLake,RDepthTot,RStatusTot ) !OUT
!---------------------------------------------------------------------------------------------------------

! PURPOSE: Make lake fraction, the most probable lake depth and lake status for the polygon in lon-lat
!         (for the grid box of the atmopheric model grid approximated by the poligon in lon-lat)
! AUTHOR: Ekaterina Kourzeneva, 
!         Meteo France, 2009
! INPUT:  NP - Number of the polygon peaks
!         PeakLon - array with longitudes of the peaks, deg. dec., -180.0 ... 180.0
!         PeakLat array with latitudes of the peaks, deg. dec., -90.0 ... 90.0
!         IStat = 1 to use mode statistics(most probable value, maximum of histogram), recommended
!               = 2 to use average value
! USES: module Bitmap (Bitmap.f90, do not edit!)
! CALLS EXTERNAL SUBROUTINE(S) OR FUNCTION(S): Coor2Num (Coor2Num.f90)
!                                              inpoly (inpoly.f)
! CALLS INTERNAL SUBROUTINE: ReadLake (in this file)
! READS FILES: data for the lake depth and status (GlobalLakeDepth.dat and GlobalLakeStatus.dat)
! Be sure that all necessary files are present!!!
! OUTPUT: RFrLake - fraction of lake surface in the poligon, 0..1 
!         RDepthTot - the depth of the lake(s) in the poligon, m
!         RStatusTot - value characterizing the reliability of the information about lake depth
!                    = 0 - no lakes in the poligon
!                    = 1 - no real information about lakes, lake depth is just of the default value = 10m
!                    = 2 - no real information about lake depth, lake depth is just of the default value = 10m
!                    = 3 - there is a real information about lake depth used
!                    = 4 - there are rivers, the default value for depth is = 3m
!                    = 5 - lake depth is estimated by an expert evaluation method
!                    = 6 - lake depth is estimated by the geographical method
!                    = 7 - lake depth is estimated by the geomorphologic method
! WRITES FILES: no
!------------------------------------------------------------------------------------------------------------

 USE Bitmap, ONLY : &
  NlonB ! Number of longitude pixels of the bitmap
!------------------------------------------------------------------------------------------------------------
 
 IMPLICIT NONE
! ---------------------------------------------------------------------------------------------------------------
 
 INTEGER, INTENT(IN) :: NP ! Number of the polygon peaks
 REAL, DIMENSION(NP),INTENT(IN) :: PeakLon, PeakLat ! Arrays to store the coordinates
                                                    ! (longitude and latitude) of the polygon peaks
 INTEGER, INTENT(IN) :: IStat ! Which statistics to choose:   = 1 for mode (most probable, maximum of histogram)
                              !                               = 2 for average
 REAL, INTENT(OUT) :: RFrLake, & ! Fraction of lake
                      RDepthTot  ! Most probable lake depth
 INTEGER, INTENT(OUT) :: RStatusTot   ! Most probable lake status                      

!---------------------------------------------------------------------------------------------------------------
 INTEGER, PARAMETER :: NGradStatus = 8 ! Number of gradations for Status
 INTEGER, PARAMETER :: NGradDepth = 21 ! Number of gradations for Depth

 REAL, DIMENSION(0:NGradStatus), PARAMETER :: BoundGradStatus = (/-99999.0, 0.5, 1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 99999.0/) 
                                                                  ! Boundaries of gradations for status
 REAL, DIMENSION(0:NGradDepth), PARAMETER :: BoundGradDepth = &
      (/-99999., 0., 2., 4., 6., 8., 12., 16., 20., 24., 30., 36., 42., 58., 82., 118., 182., 318., 482., 718., 1282., 99999.0/) 
                                                                  ! Boundaries of gradations for depth
 INTEGER, DIMENSION(NGradStatus), PARAMETER :: CentrGradStatus = (/0, 1, 2, 3, 4, 5, 6, 7/) 
                                               ! Central values for the gradations for status
 REAL, DIMENSION(NGradDepth), PARAMETER :: CentrGradDepth = &
      (/0., 1., 3., 5., 7., 10., 14., 18., 22., 27., 33., 39., 50., 70., 100., 150., 250., 400., 600., 1000., 1600./)
                                               ! Central values for the gradations for depth
 REAL, PARAMETER :: SmallDummy = -99999.0 ! Small value
!------------------------------------------------------------------------------------------------------------------
 INTEGER, DIMENSION(NP) :: PeakLonPix, PeakLatPix ! Arrays to store the coordinates of the polygon peaks in pixel numbers
 REAL ::  North, South, West, East ! the coordinates of the circumscribed rectangular 
                                   ! in lon-lat around the polygon
 INTEGER ::  NorthPix, SouthPix, WestPix, EastPix ! the coordinates in pixels
 INTEGER(1), DIMENSION(:,:), ALLOCATABLE :: Status  ! Status of lake(s) on the bitmap in the circumscribed rectangular
 REAL, DIMENSION(:,:), ALLOCATABLE :: Depth ! Depth of lake(s) on the bitmap in the circumscribed rectangular

 REAL, DIMENSION(NGradStatus) :: PDFStatus ! PDF for Status
 REAL, DIMENSION(NGradDepth) :: PDFDepth ! PDF for Depth 

 INTEGER, DIMENSION(:), ALLOCATABLE ::  StatusCut! Status of lake(s) in the polygon in one line
 REAL, DIMENSION(:), ALLOCATABLE ::  DepthCut ! Depth of lake(s) in the the polygon in one line
 INTEGER, DIMENSION(NGradStatus) :: TNGStatus ! Total number of cases in the gradation
 INTEGER, DIMENSION(NGradDepth) :: TNGDepth ! Total number of cases in the gradation
 REAL :: FrLakeStatus, FrLakeDepth, DepthTotMode, DepthTotAve
 LOGICAL :: LInside ! If the pixel is inside the polygin
 LOGICAL :: L180 ! If the grid box crosses the 180 meridian

 REAL :: NPDF ! Normalised value 
 INTEGER :: ilon, ilat ! loop indexes
 REAL :: rilon, rilat
 INTEGER :: ipeak
 INTEGER :: ErCode
 INTEGER :: Npix, ipix, igrad
 INTEGER :: igradMode
 REAL :: MaxP ! Maximum value

 REAL, DIMENSION(:,:), ALLOCATABLE :: POLY ! to call the function "inpoly"

 LOGICAL inpoly ! External function
!----------------------------------------------------------------------------------

! Find coordinates of peaks of the circumscribed rectangular  ---------------------

 L180=.FALSE.                       
 North=MAXVAL(PeakLat)
 South=MINVAL(PeakLat)
 East=MAXVAL(PeakLon)
 West=MINVAL(PeakLon)
 IF(West.LT.-175..AND.East.GT.175.) THEN ! Check crossing the 180 meridian
  WRITE(*,*) 'Warning: crossing 180 meridian ...'
  L180=.TRUE.
  East=MINVAL(PeakLon)
  West=MAXVAL(PeakLon)
 END IF

 CALL Coor2Num(North, 2, NorthPix, ErCode)
 IF(ErCode.NE.0) STOP
 CALL Coor2Num(South, 2, SouthPix, ErCode)
 IF(ErCode.NE.0) STOP
 CALL Coor2Num(East, 1, EastPix, ErCode)
 IF(ErCode.NE.0) STOP
 CALL Coor2Num(West, 1, WestPix, ErCode)
 IF(ErCode.NE.0) STOP
 IF(NorthPix.GT.SouthPix) THEN
  write(*,*) 'Wrong boundaries of the grid square in pixels in lat '
  STOP
 END IF 
 IF(L180) THEN
  IF(WestPix.LT.EastPix) THEN
   write(*,*) 'Wrong boundaries of the grid square in pixels in lon '
   STOP
  END IF
 ELSE
  IF(WestPix.GT.EastPix) THEN
   write(*,*) 'Wrong boundaries of the grid square in pixels in lon '
   STOP
  END IF 
 END IF


 IF(.NOT.L180) THEN
  ALLOCATE(Status(WestPix:EastPix,NorthPix:SouthPix), STAT=ErCode)
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<Status>>'
  ALLOCATE(Depth(WestPix:EastPix,NorthPix:SouthPix), STAT=ErCode)
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<Depth>>'
 ELSE
  ALLOCATE(Status(WestPix-NlonB/2:NlonB/2+EastPix,NorthPix:SouthPix), STAT=ErCode)
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<Status>>'
  ALLOCATE(Depth(WestPix-NlonB/2:NlonB/2+EastPix,NorthPix:SouthPix), STAT=ErCode)
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<Depth>>'
 END IF
!-----------------------------------------------------------------------------------------------

! Read lake data
 CALL ReadLake()

! Find coordinates of peaks of the polygon
 DO ipeak=1,NP
  CALL Coor2Num(PeakLon(ipeak), 1, PeakLonPix(ipeak), ErCode)
  IF(ErCode.NE.0) STOP
  IF(L180) THEN
   IF(PeakLonPix(ipeak).GT.NlonB/2) THEN
    PeakLonPix(ipeak)=PeakLonPix(ipeak)-NlonB/2
   ELSE
    PeakLonPix(ipeak)=NlonB/2+PeakLonPix(ipeak)
   END IF
  END IF
  CALL Coor2Num(PeakLat(ipeak), 2, PeakLatPix(ipeak), ErCode)
  IF(ErCode.NE.0) STOP
 END DO
  
! Store the coordinates of polygon peaks in one array ---------------------------------------------

 ALLOCATE(POLY(NP,2),STAT=ErCode)
  IF(ErCode.NE.0) STOP 'Cant allocate the array <<POLY>>'
 POLY=0.
 DO ipeak=1,NP
  POLY(ipeak,1)=PeakLonPix(ipeak)
  POLY(ipeak,2)=-PeakLatPix(ipeak) ! negative because pixels are numerated from North to South
 END DO

 IF(.NOT.L180) THEN
  ALLOCATE(StatusCut((EastPix-WestPix+1)*(SouthPix-NorthPix+1)),STAT=ErCode )
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<StatusCut>>'
  ALLOCATE(DepthCut((EastPix-WestPix+1)*(SouthPix-NorthPix+1)),STAT=ErCode )
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<DepthCut>>'
 ELSE
  ALLOCATE(StatusCut((EastPix-WestPix+NlonB+1)*(SouthPix-NorthPix+1)),STAT=ErCode )
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<StatusCut>>'
  ALLOCATE(DepthCut((EastPix-WestPix+NlonB+1)*(SouthPix-NorthPix+1)),STAT=ErCode )
   IF(ErCode.NE.0) STOP 'Cant allocate the array <<DepthCut>>'
 END IF
! ----------------------------------------------------------------------------------------------------

! Form 1D arrays (samples) of lake values
 StatusCut=0
 DepthCut=0.
 ipix=0
 DO ilat=NorthPix,SouthPix
  IF(.NOT.L180) THEN
   DO ilon=WestPix, EastPix
    rilon=ilon
    rilat=-ilat ! negative because pixels are numerated from North to South
    LInside=inpoly(POLY,NP,rilon, rilat)
    IF(LInside) THEN
     ipix=ipix+1
     StatusCut(ipix)=Status(ilon,ilat)
     DepthCut(ipix)=Depth(ilon,ilat)    
    END IF
   END DO
  ELSE
   DO ilon=WestPix-NlonB/2, EastPix+NLonB/2
    rilon=ilon
    rilat=-ilat ! negative because pixels are numerated from North to South
    LInside=inpoly(POLY,NP,rilon, rilat)
    IF(LInside) THEN
     ipix=ipix+1
     StatusCut(ipix)=Status(ilon,ilat)
     DepthCut(ipix)=Depth(ilon,ilat)    
    END IF
   END DO
  END IF
 END DO
 NPix=ipix

! Free memory
 DEALLOCATE(POLY) 
 DEALLOCATE(Status)
 DEALLOCATE(Depth) 

! Make PDF for status 
 PDFStatus=0.
 TNGStatus=0.
 DO ipix=1,NPix
  DO igrad=1,NGradStatus
   IF(StatusCut(ipix).GT.BoundGradStatus(igrad-1).AND.StatusCut(ipix).LE.BoundGradStatus(igrad)) THEN
    TNGStatus(igrad)=TNGStatus(igrad)+1
    EXIT
   END IF
  END DO
 END DO
 PDFStatus=REAL(TNGStatus)/REAL(NPix) 

! Make PDF for depth
 PDFDepth=0.
 TNGDepth=0.
 DO ipix=1,NPix
  DO igrad=1,NGradDepth
   IF(DepthCut(ipix).GT.BoundGradDepth(igrad-1).AND.DepthCut(ipix).LE.BoundGradDepth(igrad)) THEN
    TNGDepth(igrad)=TNGDepth(igrad)+1
    EXIT
   END IF
  END DO
 END DO
 PDFDepth=REAL(TNGDepth)/REAL(NPix)

 DEALLOCATE(StatusCut)
 DEALLOCATE(DepthCut)

! Make statistics -----------------------------------------------------------------------------------------

 FrLakeStatus=SUM(PDFStatus(2:NGradStatus))
 FrLakeDepth=SUM(PDFDepth(2:NGradDepth))
 RFrLake=(FrLakeStatus+FrLakeDepth)/2.

 MaxP=SmallDummy
 igradMode=2
 DO igrad=2,NGradStatus ! Calculate maximum - do not trust MAXLOC function here
  NPDF=PDFStatus(igrad)/(BoundGradDepth(igrad)-BoundGradDepth(igrad-1))
  IF(NPDF.GT.MaxP) THEN
   MaxP=NPDF
   igradMode=igrad
  END IF
 END DO
 IF(FrLakeStatus.GT.0) THEN
  RStatusTot = CentrGradStatus(igradMode)
 ELSE
  RStatusTot=0
 END IF    

 MaxP=SmallDummy
 igradMode=2
 DO igrad=2,NGradDepth ! Calculate maximum - do not trust MAXLOC function here
  NPDF=PDFDepth(igrad)/(BoundGradDepth(igrad)-BoundGradDepth(igrad-1))
  IF(NPDF.GT.MaxP) THEN
   MaxP=NPDF
   igradMode=igrad
  END IF
 END DO
 IF(FrLakeDepth.GT.0) THEN
  DepthTotMode = CentrGradDepth(igradMode)    
 ELSE
  DepthTotMode = 0.
 END IF 

 DepthTotAve=0.
 DO igrad=2,NGradDepth
  DepthTotAve=DepthTotAve+CentrGradDepth(igrad)*PDFDepth(igrad)
 END DO
 IF(RFrLake.LT.0.00001) THEN
  DepthTotAve=0.
 ELSE
  DepthTotAve=DepthTotAve/FrLakeDepth
 END IF

 SELECT CASE(IStat)
  CASE(1) 
   RDepthTot=DepthTotMode
  CASE(2)
   RDepthTot=DepthTotAve
  CASE DEFAULT
   WRITE(*,*) 'Wrong choice of statistics'
   STOP
 END SELECT
! --------------------------------------------------------------------------------------------------------------

! --------------------------------------------------------------------------------------------------------------
! --------------------------------------------------------------------------------------------------------------
CONTAINS

! --------------------------------------------------------------------------------------------------------------
! --------------------------------------------------------------------------------------------------------------

 SUBROUTINE ReadLake() 
! --------------------------------------------------------------------------------------------------------------
!
! PURPOSE: Reading the lake data to the grid square
! AUTHOR: Ekaterina Kourzeneva,
!         Meteo-France, 2009
! ---------------------------------------------------------------------------------------------------------------

  IMPLICIT NONE
! ---------------------------------------------------------------------------------------------------------------

  INTEGER(1), DIMENSION(NLonB) :: LonPix1 ! Pixels of status data along the latitude
  INTEGER(2), DIMENSION(NLonB) :: LonPix2 ! Pixels of depth data along the latitude
!  ---------------------------------------------------------------------------------------------------------------

  OPEN(91,file='GlobalLakeStatus.dat', FORM='unformatted', ACCESS='direct',RECL=NLonB)
  OPEN(92,file='GlobalLakeDepth.dat', FORM='unformatted', ACCESS='direct',RECL=NLonB*2)

  DO ilat=NorthPix,SouthPix
   READ(91,REC=ilat) LonPix1
   READ(92,REC=ilat) LonPix2
   IF(.NOT.L180) THEN
    DO ilon=WestPix, EastPix
     Status(ilon,ilat)=LonPix1(ilon)
     Depth(ilon,ilat)=LonPix2(ilon)/10.
    END DO     
   ELSE
    DO ilon=WestPix,NLonB
     Status(ilon-NlonB/2,ilat)=LonPix1(ilon)
     Depth(ilon-NlonB/2,ilat)=LonPix2(ilon)/10.
    END DO
    DO ilon=1,EastPix
     Status(NlonB/2+ilon,ilat)=LonPix1(ilon)
     Depth(NlonB/2+ilon,ilat)=LonPix2(ilon)/10.     
    END DO
   END IF
  END DO

  CLOSE(91)
  CLOSE(92)   

 END SUBROUTINE ReadLake
! ------------------------------------------------------------------------------------------------------------ 
! ------------------------------------------------------------------------------------------------------------ 

END SUBROUTINE ProjectLake
! ------------------------------------------------------------------------------------------------------------ 
! ------------------------------------------------------------------------------------------------------------ 
