!c !--------------------------------------------------------------------------
!c !                            INPOLY
!c !   Function to tell if a point is inside a polygon or not.
!c !--------------------------------------------------------------------------
!c !   Copyright (c) 1995-1996 Galacticomm, Inc.  Freeware source code.
!c !
!c !   Please feel free to use this source code for any purpose, commercial
!c !   or otherwise, as long as you don't restrict anyone else's use of
!c !   this source code.  Please give credit where credit is due.
!c !
!c !   Point-in-polygon algorithm, created especially for World-Wide Web
!c !   servers to process image maps with mouse-clickable regions.
!c !
!c !   Home for this file:  http://www.gcomm.com/develop/inpoly.c
!c !
!c !                                       6/19/95 - Bob Stein & Craig Yap
!c !                                       stein@gcomm.com
!c !                                       craig@cse.fau.edu
!c !--------------------------------------------------------------------------
!c !   Modified by:
!c !   Aris Gerakis, apr. 1998: 1.  translated to Fortran
!c !                            2.  made it work with real coordinates
!c !                            3.  now resolves the case where point falls
!c !                                on polygon border.
!c !   Aris Gerakis, nov. 1998: Fixed error caused by hardware arithmetic
!c !   Aris Gerakis, july 1999: Now all borderline cases are valid
!c !   Ekaterina Kourzeneva, dec. 2009: Polygon may be with more than 7 peaks
!c !                                    updated for fortran 90
!c !--------------------------------------------------------------------------
!c !   Glossary:
!c !   function inpoly: true=inside, false=outside (is target point inside
!c !                    a 2D polygon?)
!c !   poly(*,2):  polygon points, [0]=x, [1]=y
!c !   npoints: number of points in polygon
!c !   xt: x (horizontal) of target point
!c !   yt: y (vertical) of target point
!c !--------------------------------------------------------------------------

      function inpoly (poly, npoints, xt, yt)

      implicit none

!c Declare arguments:
      logical inpoly
      integer, intent(in):: npoints
!c_ek      real    poly(7, 2), xt, yt
      real, dimension(npoints,2), intent(in) ::    poly
      real, intent(in) :: xt, yt

!c Declare local variables:

      real    xnew, ynew, xold, yold, x1, y1, x2, y2
      integer i
      logical inside, on_border

      inside = .false.
      on_border = .false.

      if (npoints .lt. 3)  then
         inpoly = .false.
         return
      end if

      xold = poly(npoints,1)
      yold = poly(npoints,2)

      do i = 1 , npoints
         xnew = poly(i,1)
         ynew = poly(i,2)

         if (xnew .gt. xold)  then
            x1 = xold
            x2 = xnew
            y1 = yold
            y2 = ynew
         else
            x1 = xnew
            x2 = xold
            y1 = ynew
            y2 = yold
         end if

!c  ! The outer IF is the 'straddle' test and the 'vertical border' test.
!c  ! The inner IF is the 'non-vertical border' test and the 'north' test.  

!c  ! The first statement checks whether a north pointing vector crosses  
!c  ! (stradles) the straight segment.  There are two possibilities, depe-
!c  ! nding on whether xnew < xold or xnew > xold.  The '<' is because edge 
!c  ! must be "open" at left, which is necessary to keep correct count when 
!c  ! vector 'licks' a vertix of a polygon.  

         if ((xnew .lt. xt .and. xt .le. xold) .or. &
           (.not. xnew .lt. xt .and.  .not. xt .le. xold)) then
!c The test point lies on a non-vertical border:
            if ((yt-y1)*(x2-x1) .eq. (y2-y1)*(xt-x1)) then
               on_border = .true. 
!c Check if segment is north of test point.  If yes, reverse the 
!c    ! value of INSIDE.  The +0.001 was necessary to avoid errors due   
!c    ! arithmetic (e.g., when clay = 98.87 and sand = 1.13):   
            elseif ((yt-y1)*(x2-x1) .lt. (y2-y1)*(xt-x1) + 0.001) then
               inside = .not.inside ! cross a segment
            endif
!c  ! This is the rare case when test point falls on vertical border or  
!c  ! left edge of non-vertical border. The left x-coordinate must be  
!c  ! common.  The slope requirement must be met, but also point must be
!c  ! between the lower and upper y-coordinate of border segment.  There 
!c  ! are two possibilities,  depending on whether ynew < yold or ynew > 
!c  ! yold:
         elseif ((xnew .eq. xt .or. xold .eq. xt) .and. &
               (yt-y1)*(x2-x1) .eq. (y2-y1)*(xt-x1) .and. &
               ((ynew .le. yt .and. yt .le. yold) .or. &
           (.not. ynew .lt. yt .and. .not. yt .lt. yold))) then
            on_border = .true. 
         endif

         xold = xnew
         yold = ynew

      enddo

!c! If test point is not on a border, the function result is the last state 
!c! of INSIDE variable.  Otherwise, INSIDE doesn't matter.  The point is
!c! inside the polygon if it falls on any of its borders:

      if (.not. on_border) then
         inpoly = inside
      else
         inpoly = .true.
      endif

      return
      
      end function inpoly
